@@ -2,7 +2,7 @@
 #' 
 #'   Lookup table of information about cluster processes and Cox processes
 #'
-#'   $Revision: 1.57 $ $Date: 2023/01/02 02:57:50 $
+#'   $Revision: 1.61 $ $Date: 2023/02/25 10:39:10 $
 #'
 #'   Information is extracted by calling
 #'             spatstatClusterModelInfo(<name>)
@@ -867,9 +867,9 @@ resolve.vargamma.shape <- function(...,
   cmod <- dots$covmodel
   model <- cmod$model %orifnull% dots$model %orifnull% "exponential"
   margs <- NULL
-  shortcut <- existsSpatstatVariable("RFshortcut") && isTRUE(getSpatstatVariable("RFshortcut"))
+  shortcut <- isTRUE(attr(model, "RFshortcut") %orifnull% (existsSpatstatVariable("RFshortcut") && isTRUE(getSpatstatVariable("RFshortcut"))))
   if((model %in% c("exponential", "fastGauss", "fastStable", "fastGencauchy")) ||
-     (shortcut && (model %in% c("gauss", "stable", "cauchy")))) {
+     (shortcut && (model %in% c("gauss", "stable", "gencauchy")))) {
     ## avoid RandomFields package
     ## extract shape parameters and validate them
     switch(model,
@@ -899,6 +899,7 @@ resolve.vargamma.shape <- function(...,
              })
            })
   } else {
+    ## use RandomFields package
     ## get the 'model generator' 
     modgen <- getRandomFieldsModelGen(model)
     attr(model, "modgen") <- modgen
@@ -918,6 +919,7 @@ resolve.vargamma.shape <- function(...,
       stop("Anisotropic covariance models cannot be used",
            call.=FALSE)
   }
+  attr(model, "RFshortcut") <- shortcut
   out$margs <- margs
   out$model <- model
   out$covmodel <- list(type="Covariance", model=model, margs=margs)
@@ -961,7 +963,7 @@ resolve.vargamma.shape <- function(...,
     ## 'par' is in native format
     if(any(par <= 0))
       return(rep.int(Inf, length(rvals)))
-    shortcut <- existsSpatstatVariable("RFshortcut") && isTRUE(getSpatstatVariable("RFshortcut"))
+    shortcut <- isTRUE(attr(model, "RFshortcut") %orifnull% (existsSpatstatVariable("RFshortcut") && getSpatstatVariable("RFshortcut")))
     if((model %in% c("exponential", "fastGauss", "fastStable", "fastGencauchy")) || 
        (shortcut && (model %in% c("gauss", "stable", "cauchy")))) {
       ## For efficiency and to avoid need for RandomFields package
@@ -1016,9 +1018,9 @@ resolve.vargamma.shape <- function(...,
     ## 'par' is in native format
     if(any(par <= 0))
       return(rep.int(Inf, length(rvals)))
-    shortcut <- existsSpatstatVariable("RFshortcut") && isTRUE(getSpatstatVariable("RFshortcut"))
+    shortcut <- isTRUE(attr(model, "RFshortcut") %orifnull% (existsSpatstatVariable("RFshortcut") && getSpatstatVariable("RFshortcut")))
     if((model %in% c("exponential", "fastGauss", "fastStable", "fastGencauchy")) ||
-       (shortcut && (model %in% c("gauss", "stable", "cauchy")))) {
+       (shortcut && (model %in% c("gauss", "stable", "gencauchy")))) {
       ## For efficiency and to avoid need for RandomFields package
       switch(model,
              exponential = {
@@ -1058,11 +1060,15 @@ resolve.vargamma.shape <- function(...,
   },
   Dpcf= function(par,rvals, ..., model){
     ## 'par' is in native format
-    if(!identical(model, "exponential")) {
-      stop("Gradient of the pcf not available for this model.")
-    } 
-    dsigma2 <- exp(-rvals/par[2L]) * exp(par[1L]*exp(-rvals/par[2L]))
-    dalpha <- rvals * par[1L] * exp(-rvals/par[2L]) * exp(par[1L]*exp(-rvals/par[2L]))/par[2L]^2
+    if(!(model %in% c("exponential", "stable")))
+      stop("Gradient of the pcf is not available for this model")
+    if(model=="exponential") {
+      dsigma2 <- exp(-rvals/par[2L]) * exp(par[1L]*exp(-rvals/par[2L]))
+      dalpha <- rvals * par[1L] * exp(-rvals/par[2L]) * exp(par[1L]*exp(-rvals/par[2L]))/par[2L]^2
+    } else if(model=="stable"){
+      dsigma2 <- exp(-sqrt(rvals/par[2L])) * exp(par[1L]*exp(-sqrt(rvals/par[2L])))
+      dalpha <- sqrt(rvals/par[2L]^3) * par[1L] * exp(-sqrt(rvals/par[2L])) * exp(par[1L] * exp(-sqrt(rvals/par[2L])))
+    }
     out <- rbind(dsigma2, dalpha)
     rownames(out) <- c("sigma2","alpha")
     return(out)
